==> asn.1_4.12.tar.gz: asn.1-4.12/runtime/sequence.lisp <==
;;;; -*- Mode: Lisp -*-
;;;; $Id: sequence.lisp 510 2008-09-19 14:33:11Z binghe $

(in-package :asn.1)

(defmethod plain-value ((object sequence))
  (coerce object 'vector))

(defmethod ber-equal ((a sequence) (b sequence))
  (every #'ber-equal a b))

;;; used by pdu.lisp
(defun ber-encode-list (list)
  (apply #'concatenate 'vector (mapcar #'ber-encode list)))

;; for non-null sequence
(defmethod ber-encode ((value sequence))
  "Encode any VECTOR and non-nil LIST into ASN.1 sequence"
  (let ((sub-encode (apply #'concatenate 'vector
                           (map 'list #'ber-encode value))))
    (concatenate 'vector
                 (ber-encode-type 0 1 16)
                 (ber-encode-length (length sub-encode))
                 sub-encode)))

(defmethod ber-decode-value ((stream stream) (type (eql :sequence)) length)
  (declare (type fixnum length) (ignore type))
  "Decode ASN.1 sequence data and return a VECTOR"
  (if (zerop length) #()
    ;; for non-null sequence
    (labels ((iter (length-left acc)
               (if (zerop length-left)
                 (nreverse acc)
                 (multiple-value-bind (sub-type sub-type-length)
                     (ber-decode-type stream)
                   (multiple-value-bind (sub-length sub-length-length)
                       (ber-decode-length stream)
                     (iter (- length-left
                              sub-type-length
                              sub-length-length
                              sub-length)
                           (cons (ber-decode-value stream sub-type sub-length) acc)))))))
      (let ((temp (coerce (iter length nil) 'vector)))
        ;; (format t "sequence: ~A~%" temp)
        temp))))

(eval-when (:load-toplevel :execute)
  (install-asn.1-type :sequence 0 1 16))

==> asn.1_4.14.tar.gz: asn.1-4.14/runtime/sequence.lisp <==
;;;; -*- Mode: Lisp -*-
;;;; $Id: sequence.lisp 609 2008-11-13 15:44:49Z binghe $

(in-package :asn.1)

(defmethod plain-value ((object sequence))
  (coerce object 'vector))

(defmethod ber-equal ((a sequence) (b sequence))
  (every #'ber-equal a b))

;;; used by pdu.lisp
(defun ber-encode-list (list)
  (apply #'concatenate 'vector (mapcar #'ber-encode list)))

;; for non-null sequence
(defmethod ber-encode ((value sequence))
  "Encode any VECTOR and non-nil LIST into ASN.1 sequence"
  (let ((sub-encode (apply #'concatenate 'vector
                           (map 'list #'ber-encode value))))
    (concatenate 'vector
                 (ber-encode-type 0 1 16)
                 (ber-encode-length (length sub-encode))
                 sub-encode)))

(defmethod ber-decode-value ((stream stream) (type (eql :sequence)) length)
  (declare (type fixnum length) (ignore type))
  "Decode ASN.1 sequence data and return a VECTOR"
  (if (zerop length) #()
    ;; for non-null sequence
    (labels ((iter (length-left acc)
               (if (zerop length-left)
                 (nreverse acc)
                 (multiple-value-bind (sub-type sub-type-length sub-type-bytes)
                     (ber-decode-type stream)
                   (multiple-value-bind (sub-length sub-length-length sub-length-bytes)
                       (ber-decode-length stream)
                     (iter (- length-left
                              sub-type-length
                              sub-length-length
                              sub-length)
                           (cons (if sub-type (ber-decode-value stream sub-type sub-length)
                                   (ber-decode-value stream sub-type
                                                     (cons sub-length
                                                           (append sub-type-bytes
                                                                   sub-length-bytes))))
                                 acc)))))))
      (let ((temp (coerce (iter length nil) 'vector)))
        ;; (format t "sequence: ~A~%" temp)
        temp))))

(eval-when (:load-toplevel :execute)
  (install-asn.1-type :sequence 0 1 16))
